<?php
declare(strict_types=1);
namespace Webandco\Giphy\AssetSource;

use Neos\Flow\Annotations as Flow;
use Neos\Flow\Configuration\Exception\InvalidConfigurationException;
use Neos\Flow\ResourceManagement\ResourceManager;
use Neos\Media\Domain\Model\AssetSource\AssetProxyRepositoryInterface;
use Neos\Media\Domain\Model\AssetSource\AssetSourceInterface;
use Neos\Media\Domain\Model\AssetSource\Neos\NeosAssetProxyRepository;

class GiphyAssetSource implements AssetSourceInterface
{
    /**
     * @var string
     */
    private $assetSourceIdentifier;

    /**
     * @var NeosAssetProxyRepository
     */
    private $assetProxyRepository;

    /**
     * @Flow\Inject
     * @var ResourceManager
     */
    protected $resourceManager;

    /**
     * @var string
     */
    private $copyrightNoticeTemplate;

    /**
     * @var string
     */
    protected $iconPath;

    /**
     * @var string
     */
    protected $apiKey;

    /**
     * GiphyAssetSource constructor.
     * @param string $assetSourceIdentifier
     * @param array $assetSourceOptions
     * @throws InvalidConfigurationException
     */
    public function __construct(string $assetSourceIdentifier, array $assetSourceOptions)
    {
        $this->assetSourceIdentifier = $assetSourceIdentifier;
        $this->copyrightNoticeTemplate = $assetSourceOptions['copyrightNoticeTemplate'] ?? '';
        $this->iconPath = $assetSourceOptions['icon'] ?? '';

        if (!isset($assetSourceOptions['apiKey']) || trim($assetSourceOptions['apiKey']) === '') {
            throw new InvalidConfigurationException(sprintf('Api Key for the %s data source not set.', $this->getLabel()), 1590846378);
        }

        $this->apiKey = $assetSourceOptions['apiKey'];

    }

    /**
     * This factory method is used instead of a constructor in order to not dictate a __construct() signature in this
     * interface (which might conflict with an asset source's implementation or generated Flow proxy class).
     *
     * @param string $assetSourceIdentifier
     * @param array $assetSourceOptions
     * @return AssetSourceInterface
     * @throws InvalidConfigurationException
     */
    public static function createFromConfiguration(string $assetSourceIdentifier, array $assetSourceOptions): AssetSourceInterface
    {
        return new static($assetSourceIdentifier, $assetSourceOptions);
    }

    /**
     * A unique string which identifies the concrete asset source.
     * Must match /^[a-z][a-z0-9-]{0,62}[a-z]$/
     *
     * @return string
     */
    public function getIdentifier(): string
    {
        return $this->assetSourceIdentifier;
    }

    /**
     * @return string
     */
    public function getLabel(): string
    {
        return 'Giphy';
    }

    /**
     * @return string
     */
    public function getCopyrightNoticeTemplate(): string
    {
        return $this->copyrightNoticeTemplate;
    }

    /**
     * @return AssetProxyRepositoryInterface
     */
    public function getAssetProxyRepository(): AssetProxyRepositoryInterface
    {
        if ($this->assetProxyRepository === null) {
            $this->assetProxyRepository = new GiphyAssetProxyRepository($this);
        }

        return $this->assetProxyRepository;
    }

    /**
     * @return bool
     */
    public function isReadOnly(): bool
    {
        return true;
    }

    /**
     * Returns the resource path to Assetsources icon
     *
     * @return string
     */
    public function getIconUri(): string
    {
        return $this->resourceManager->getPublicPackageResourceUriByPath($this->iconPath);
    }

    /**
     * @return string
     */
    public function getApiKey(): string {
        return $this->apiKey;
    }

    /**
     * @return string
     */
    public function getDescription(): string
    {
        return 'https://giphy.com/';
    }


}
